#!/usr/bin/env python

"""
Moin serialiser support.

Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class Serialiser:

    "General serialisation support."

    format = None # defined by subclasses

    def __init__(self, output, formats=None, linker=None):

        """
        Initialise the serialiser with an 'output' context, an optional
        'formats' mapping from names to serialiser classes, and an optional
        'linker' object for translating links.
        """

        self.output = output
        self.formats = formats
        self.linker = linker

        # Initialise a callable for use in serialisation.

        self.out = output.out

        # Initialisation of any other state.

        self.init()

    def init(self):

        "Initialisation method to be overridden by subclasses."

        pass

    def __repr__(self):
        return "%s(%r, %r, %r)" % (self.__class__.__name__, self.output,
                                   self.formats, self.linker)

    def get_serialiser(self, format):

        """
        Return a serialiser for the given 'format'. Return self if no suitable
        serialiser can be obtained.
        """

        cls = self.formats and self.formats.get(format)
        if cls:
            return self.instantiate(cls)
        else:
            return self

    def get_output(self):

        "Return the output as a string."

        return self.output.to_string()

    def instantiate(self, cls):

        """
        Instantiate 'cls' and return the result if 'cls' is a different class to
        this instance. Otherwise, return this instance.
        """

        if cls is self.__class__:
            return self
        else:
            return cls(self.output, self.formats, self.linker)

def escape_attr(s):

    "Escape XML document attribute."

    return escape_text(s).replace("'", "&apos;").replace('"', "&quot;")

def escape_text(s):

    "Escape XML document text."

    return s.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")

# vim: tabstop=4 expandtab shiftwidth=4
